do
    -- These two fixes should allow us to drop LibUIDropDownMenu and use native menus.  It would fix #12 without having to fork LibUIDropDownMenu.

    -- HonorFrameLoadTaint workaround - https://www.townlong-yak.com/bugs/afKy4k-HonorFrameLoadTaint
    if (UIDROPDOWNMENU_VALUE_PATCH_VERSION or 0) < 2 then
        UIDROPDOWNMENU_VALUE_PATCH_VERSION = 2
        hooksecurefunc("UIDropDownMenu_InitializeHelper", function()
            if UIDROPDOWNMENU_VALUE_PATCH_VERSION ~= 2 then
                return
            end
            for i=1, UIDROPDOWNMENU_MAXLEVELS do
                for j=1, UIDROPDOWNMENU_MAXBUTTONS do
                    local b = _G["DropDownList" .. i .. "Button" .. j]
                    if not (issecurevariable(b, "value") or b:IsShown()) then
                        b.value = nil
                        repeat
                            j, b["fx" .. j] = j+1
                        until issecurevariable(b, "value")
                    end
                end
            end
        end)
    end

    -- DisplayModeCommunitiesTaint workaround - https://www.townlong-yak.com/bugs/Kjq4hm-DisplayModeCommunitiesTaint
    if (UIDROPDOWNMENU_OPEN_PATCH_VERSION or 0) < 1 then
        UIDROPDOWNMENU_OPEN_PATCH_VERSION = 1
        hooksecurefunc("UIDropDownMenu_InitializeHelper", function(frame)
            if UIDROPDOWNMENU_OPEN_PATCH_VERSION ~= 1 then
                return
            end
            if UIDROPDOWNMENU_OPEN_MENU and UIDROPDOWNMENU_OPEN_MENU ~= frame
            and not issecurevariable(UIDROPDOWNMENU_OPEN_MENU, "displayMode") then
                UIDROPDOWNMENU_OPEN_MENU = nil
                local t, f, prefix, i = _G, issecurevariable, " \0", 1
                repeat
                    i, t[prefix .. i] = i + 1
                until f("UIDROPDOWNMENU_OPEN_MENU")
            end
        end)
    end
end



local addonName, addon = ...

-- functions
local CreateFrame = CreateFrame
local EasyMenu = EasyMenu
-- constants
-- frames
local UIParent = UIParent
-- libs

local contextMenu = {
    menuFrame = CreateFrame("frame", "BrokerKeystonesContextMenuFrame", UIParent, "UIDropDownMenuTemplate"),
    item = {
        Action = function(self, label, func)
            return {text = label, notCheckable = true, func = func}
        end,
        Checkbox = function(self, label, checked, func)
            return {text = label, keepShownOnClick = true, isNotRadio = true, checked = checked, func = func}
        end,
        Separator = function(self)
            return {text = " ", disabled = true, notCheckable = true}
        end,
        SubMenu = function(self, title, menuList)
            return {text = title, notCheckable = true, hasArrow = true, menuList = menuList}
        end,
        Title = function(self, title)
            return {text = title, isTitle = true, notCheckable = true}
        end,
        OptionToggle = function(self, label, key)
            return self:Checkbox(
                label,
                function() return addon.options[key] end,
                function() addon.options[key] = not addon.options[key] end
            )
        end,
    },
}

function contextMenu:Show(menu, anchor)
    EasyMenu(menu, self.menuFrame, anchor or "cursor", 0, 0, "MENU")
end

addon:RegisterCallback(addon.EVENT_BROKER_INITIALISED, function(_, broker)
    broker.contextMenu = contextMenu
end)
