local addonName, addon = ...

-- functions
local format = string.format
local pairs = pairs
local tinsert = table.insert
local type = type
local unpack = unpack
-- constants
-- frames
-- libs

local private = {
    broker = nil,
    optionName = "useClassicTooltip",
    tooltipLines = {},
}

function private:GetTooltipLines()
    wipe(private.tooltipLines)

    local charactersWithKeystones = addon:GetCharacters(function(_, c)
        return c.keystone ~= nil
    end)
    local numKeystones = addon:TableLength(charactersWithKeystones)

    local charactersWithRecords = addon.options.showRecords
        and addon:GetCharacters(function(_, c) return c.record ~= nil end)
        or {}
    local numRecords = addon:TableLength(charactersWithRecords)

    if numKeystones < 1 and numRecords < 1 then
        tinsert(private.tooltipLines, {"No Keystones", nil, 1, 1, 1})
        return private.tooltipLines
    end

    tinsert(private.tooltipLines, {
        "Character",
        format("%d |cffFFFFFF%s|r", numKeystones, numKeystones == 1 and "Keystone" or "Keystones"),
        1, 1, 1,
    })
    if not addon.options.showRecords then tinsert(private.tooltipLines, " ") end

    -- merge the two tables
    local characters = {}
    if numKeystones > 0 then
        for k, v in pairs(charactersWithKeystones) do
            characters[k] = v
        end
    end
    if numRecords > 0 then
        for k, v in pairs(charactersWithRecords) do
            characters[k] = v
        end
    end

    for key, character in pairs(characters) do
        if addon.options.showRecords then tinsert(private.tooltipLines, " ") end
        tinsert(private.tooltipLines, {
            addon:GetCharacterDisplayName(key, true),
            character.keystone
                and addon:GetKeystoneDisplayText(character.keystone, true)
                or format("|cff666666%s|r", "(no keystone)"),
            1.0, 1.0, 1.0,
            1.0, 1.0, 1.0,
        })
        if addon.options.showRecords and character.record then
            local mapId, level = unpack(character.record)
            -- todo: shorthand
            local instanceName = addon:GetMapName(mapId)
            tinsert(private.tooltipLines, {
                " ",
                format("Best: %s %d", instanceName, level),
                nil, nil, nil,
                0.7, 0.7, 0.7,
            })
        end
    end

    return private.tooltipLines
end

local function OnTooltipShow(tooltip)
    local lines = private:GetTooltipLines()

    for _, line in pairs(lines) do
        if type(line) == "string" then
            line = {line}
        end

        tooltip:AddDoubleLine(unpack(line))
    end
end

addon:RegisterCallback(addon.EVENT_BROKER_INITIALISED, function(_, broker)
    private.broker = broker
    if addon.options.useClassicTooltip then
        broker.OnTooltipShow = OnTooltipShow
    end
end)

addon:RegisterCallback(addon.EVENT_OPTION_CHANGED, function(_, key, newValue, oldValue)
    if key ~= private.optionName or not private.broker then return end

    if newValue then
        private.broker.OnTooltipShow = OnTooltipShow
    elseif private.broker.OnTooltipShow == OnTooltipShow then
        private.broker.OnTooltipShow = nil
    end
end)
