local _, addon = ...

-- tables
local string = string
-- functions
local print, select, tostring, type = print, select, tostring, type
local format = string.format
-- constants
-- frames
-- libs

addon.EVENT_BAG_SCAN_STARTED = "BAG_SCAN_STARTED"
addon.EVENT_BAG_SCAN_FINISHED = "BAG_SCAN_FINISHED"
addon.EVENT_CHARACTER_ADDED = "CHARACTER_ADDED"
addon.EVENT_CHARACTER_CHANGED = "CHARACTER_CHANGED"
addon.EVENT_CHARACTER_REMOVED = "CHARACTER_REMOVED"
addon.EVENT_KEYSTONE = "KEYSTONE"
addon.EVENT_GUILD_INFO_CHANGED = "GUILD_INFO_CHANGED"
addon.EVENT_PLAYER_INFO_CHANGED = "PLAYER_INFO_CHANGED"
addon.EVENT_OPTION_CHANGED = "OPTION_CHANGED"
addon.EVENT_RECORD = "RECORD"
addon.EVENT_RECORD_SCAN_STARTED = "RECORD_SCAN_STARTED"
addon.EVENT_RECORD_SCAN_FINISHED = "RECORD_SCAN_FINISHED"

addon.EVENT_INITIALISED = "INITIALISED"
addon.EVENT_CORE_INITIALISED = "CORE_INITIALISED"
addon.EVENT_BROKER_INITIALISED = "BROKER_INITIALISED"

function addon:Print(...)
    if select("#", ...) < 1 then return end

    local args = {...}
    args[1] = "|cff6699CCBK|r " .. args[1]
    print(format(unpack(args)))
end

function addon:Debug(...)
    if select("#", ...) < 1 or not self.db or not self.db.options or self.db.options.debug ~= true then return end

    local args = {...}
    args[1] = "|cffFF9900Debug|r " .. args[1]
    return self:Print(unpack(args))
end

function addon:Initialise(db)
    self.db = db

    self:Debug("Initialising.")

    self.db.characters = self.db.characters or {}
    -- migrate old "pools" system into new db format
    if self.db.pools ~= nil then
        if self.db.pools.player ~= nil then
            self.db.characters = self.db.pools.player
            self.db.pools.player = nil
        end
        self.db.pools = nil
    end

    local key = self:GetCharacterKey()
    local playerInfo = self:GetPlayerInfo()
    self.db.characters[key] = self.db.characters[key] or playerInfo

    --[[ Disable this, because I don't know if this function returns true immediately or after an event.
    if not C_MythicPlus.IsMythicPlusActive() then
        for _, character in pairs(self.db.characters) do
            character.previousBestRun = nil
        end
    end
    ]]

    self.player = self.db.characters[key]
    for k, v in pairs(playerInfo) do
        if self.player[k] ~= playerInfo[k] then
            self:Debug("Player %s has changed (%s -> %s); updating database.", k, self.player[k], playerInfo[k])
            self.player[k] = playerInfo[k]
        end
    end

    if self.db.options and self.db.options.debug then
        _G.Broker_Keystones = self
    end

    self:Fire(self.EVENT_CORE_INITIALISED)
end

addon:RegisterCallback(addon.EVENT_INITIALISED, function()
    addon:Debug("Ready.")
end)
