local _, addon = ...

-- tables
local string = string
local table = table
local C_ChallengeMode = C_ChallengeMode
local C_MythicPlus = C_MythicPlus
-- functions
local pairs = pairs
local format = string.format
local tconcat = table.concat
local tinsert = table.insert
local wipe = wipe
local CreateFrame = CreateFrame
local C_ChallengeMode_GetAffixInfo = C_ChallengeMode.GetAffixInfo
local C_MythicPlus_GetCurrentAffixes = C_MythicPlus.GetCurrentAffixes
-- constants
-- frames
-- libs

local AFFIX_LEVEL_START = 4
local AFFIX_LEVEL_INCREMENT = 3

addon.affixes = {}
addon.affixesStr = nil
addon.affixesStrConcat = {}

local function onEvent(self, event, ...)
    wipe(addon.affixes)
    addon.affixesStr = nil
    wipe(addon.affixesStrConcat)

    local affixes = C_MythicPlus_GetCurrentAffixes()
    if not affixes then return end

    local level = AFFIX_LEVEL_START
    for _, affixId in pairs(affixes) do
        local name, desc, fileData = C_ChallengeMode_GetAffixInfo(affixId)
        tinsert(addon.affixes, {
            id = affixId,
            name = name,
            desc = desc,
            fileData = fileData,
            level = level,
        })
        level = level + AFFIX_LEVEL_INCREMENT
    end

    for _, affix in ipairs(addon.affixes) do
        tinsert(addon.affixesStrConcat, format("|T%s:0:0:0:-1|t %s%s|r", affix.fileData, addon:GetKeystoneLevelColour(affix.level), affix.name))
    end

    addon.affixesStr = tconcat(addon.affixesStrConcat, " ")
end

local f = CreateFrame("frame")
f:SetScript("OnEvent", onEvent)
f:RegisterEvent("MYTHIC_PLUS_CURRENT_AFFIX_UPDATE")

-- just in case some addon before us has called it and the info is already there
if C_MythicPlus_GetCurrentAffixes() then
    onEvent(f, "MYTHIC_PLUS_CURRENT_AFFIX_UPDATE")
end
