local _, addon = ...

-- tables
local C_ChallengeMode = C_ChallengeMode
local C_MythicPlus = C_MythicPlus
local C_Timer = C_Timer
-- functions
local format = format
local ipairs = ipairs
local unpack = unpack
local C_ChallengeMode_GetMapTable = C_ChallengeMode.GetMapTable
local C_ChallengeMode_RequestLeaders = C_ChallengeMode.RequestLeaders
local C_MythicPlus_GetWeeklyBestForMap = C_MythicPlus.GetWeeklyBestForMap
local C_MythicPlus_GetWeeklyChestRewardLevel = C_MythicPlus.GetWeeklyChestRewardLevel
local C_MythicPlus_RequestMapInfo = C_MythicPlus.RequestMapInfo
local C_MythicPlus_RequestRewards = C_MythicPlus.RequestRewards
local C_Timer_After = C_Timer.After
local C_Timer_NewTimer = C_Timer.NewTimer
-- constants
-- frames
-- libs

addon:RegisterCallback(addon.EVENT_RECORD_SCAN_STARTED, function()
    addon:Debug("Scanning for character best runs...")
end)

addon:RegisterCallback(addon.EVENT_RECORD_SCAN_FINISHED, function(_, bestMapId, bestLevel, bestDurationSec)
    if bestMapId then
        addon:Debug("Character's best run is %s %d (in %ds).", addon:GetMapName(bestMapId), bestLevel, bestDurationSec)
    else
        addon:Debug("No best run found.")
    end

    addon:SetCharacterRecord(addon:GetCharacterKey(), bestMapId, bestLevel, bestDurationSec)
end)

addon:RegisterCallback(addon.EVENT_INITIALISED, function()
    local scanDelay = 2
    local scanTimer
    local function scanCallback(_, event, ...)
        if event == "CHALLENGE_MODE_COMPLETED" then
            C_MythicPlus_RequestMapInfo()
            C_MythicPlus_RequestRewards()
            return
        end

        addon:Fire(addon.EVENT_RECORD_SCAN_STARTED)

        local bestMapId, bestLevel, bestDurationSec

        --[[ needs verification ]]
        if event == "MYTHIC_PLUS_NEW_WEEKLY_RECORD" or event == "MYTHIC_PLUS_NEW_SEASON_RECORD" then
            local newMapId, newCompletionMilliseconds, newLevel = ...
            bestMapId = newMapId
            bestLevel = newLevel
            bestDurationSec = newCompletionMilliseconds / 1000

            addon:Fire(addon.EVENT_RECORD_SCAN_FINISHED, bestMapId, bestLevel, bestDurationSec)
            return
        end

        -- This sometimes returns data for the character that the player was logged on to prior to their current character.  Can't do anything about that until Blizzard looks into it.
        --[[
        local maps = C_ChallengeMode_GetMapTable()
        for _, mapId in ipairs(maps) do
            local durationSec, level = C_MythicPlus_GetWeeklyBestForMap(mapId)

            if level then
                if not bestMapId then
                    bestMapId = mapId
                    bestLevel = level
                    bestDurationSec = durationSec
                end

                if level > bestLevel
                or (level == bestLevel and durationSec > bestDurationSec)
                then
                    bestMapId = mapId
                    bestLevel = level
                    bestDurationSec = durationSec
                end
            end
        end
        ]]

        -- Due to above issue, we can't accurately determine best map ID.  So just wing it.
        local currentWeekBestLevel, weeklyRewardLevel, nextDifficultyWeeklyRewardLevel = C_MythicPlus_GetWeeklyChestRewardLevel()
        if currentWeekBestLevel and currentWeekBestLevel > 0 then
            local currentBestMapId, currentBestLevel, currentBestDurationSec = unpack(addon:GetCharacter().record or {})

            bestLevel = currentWeekBestLevel
            if currentBestLevel and currentBestLevel == currentWeekBestLevel then
                -- preserve existing run if possible
                addon:Debug("Preserving characters' best run dungeon.")
                bestMapId = currentBestMapId
                bestDurationSec = currentBestDurationSec
            else
                -- Set it to 0 and just display it as "Mythic+" instead of actual dungeon name.
                addon:Debug("Unable to accurately determine character's best run dungeon, falling back.")
                bestMapId = 0
                bestDurationSec = 0
            end
        end

        addon:Fire(addon.EVENT_RECORD_SCAN_FINISHED, bestMapId, bestLevel, bestDurationSec)
    end

    local function onEvent(self, event, ...)
        if event == "MYTHIC_PLUS_NEW_WEEKLY_RECORD" or event == "MYTHIC_PLUS_NEW_SEASON_RECORD" then
            scanCallback(self, event, ...)
            return
        end

        if scanTimer then scanTimer:Cancel() end

        local args = {...}
        scanTimer = C_Timer_NewTimer(scanDelay, function() scanCallback(self, event, unpack(args)) end)
    end

    local f = CreateFrame("frame")
    f:SetScript("OnEvent", onEvent)
    f:RegisterEvent("CHALLENGE_MODE_COMPLETED")
    -- f:RegisterEvent("CHALLENGE_MODE_LEADERS_UPDATE")
    f:RegisterEvent("CHALLENGE_MODE_MAPS_UPDATE")
    -- f:RegisterEvent("CHALLENGE_MODE_RESET")
    -- f:RegisterEvent("CHALLENGE_MODE_START")
    f:RegisterEvent("MYTHIC_PLUS_NEW_SEASON_RECORD")
    f:RegisterEvent("MYTHIC_PLUS_NEW_WEEKLY_RECORD")

    C_MythicPlus_RequestMapInfo()
    C_MythicPlus_RequestRewards()

    --[[
    local maps = C_ChallengeMode_GetMapTable()
    for _, mapId in ipairs(maps) do
        C_ChallengeMode_RequestLeaders(mapId)
    end
    ]]
end)
